@@ -2,7 +2,7 @@
 #define CRATEPLUGINEU_H
 
 #include "ActionPlugin.h"
-
+#include "ltr/include/ltr030api.h"
 
 class CrateFirmeareUpdateEU;
 class QProgressDialog;
@@ -28,6 +28,11 @@ private slots:
     void opProgress(QString stage, unsigned done);
     void opFinished(int err);
 private:
+    void resetCrate(LtrManagerEnv *env, TLTR030 &hcrate, LtrdConnection *con, LtrCrateInfo   *crate);
+    void ipReconnect(LtrManagerEnv *env, LtrdConnection *con, LtrCrateInfo   *crate);
+    static bool crateSupportEthFwUpdate(LtrCrateInfo &crate);
+    static bool crateSupportReset(LtrCrateInfo &crate);
+
     CrateFirmeareUpdateEU *m_fwUpdater;
     QThread *m_fwThread;
     LtrManagerEnv *m_env;
