@@ -21,7 +21,7 @@ LtrdConnection::LtrdConnection(QHostAddress addr, bool autocon, quint16 port, QO
     qRegisterMetaType<LtrdConnection::connStatus>("LtrdConnection::connStatus");
     qRegisterMetaType<LtrConEvent>("LtrConEvent");
     qRegisterMetaType<TLTR_CRATE_IP_ENTRY>("TLTR_CRATE_IP_ENTRY");
-    qRegisterMetaType<QSharedPointer<LtrSrvLogRec> >("QSharedPointer<LtrSrvLogRec>");
+    qRegisterMetaType<QList<QSharedPointer<LtrSrvLogRec> > >("QList<QSharedPointer<LtrSrvLogRec> >");
 
     connect(this, SIGNAL(aboutToClose()), this, SLOT(privCloseConnection()));
     connect(this, SIGNAL(aboutToStart()), SLOT(privStartConnection()));
@@ -63,6 +63,10 @@ QList<TLTR_CRATE_IP_ENTRY> LtrdConnection::ipEntryList() {
     return m_ipList;
 }
 
+bool LtrdConnection::statisticAviableForMid(WORD mid) const {
+    return (mid != LTR_MID_EMPTY) && !((mid == LTR_MID_IDENTIFYING) && !supportIndentifyModuleStat());
+}
+
 int LtrdConnection::getLogLevel(unsigned* lvl) {
     QMutexLocker lock(&m_mutex);
     return LTR_GetLogLevel(&hnd, (INT*)lvl);
@@ -155,12 +159,12 @@ void LtrdConnection::clearConnection() {
     if (LTR_IsOpened(&hnd)==LTR_OK) {
         LTR_Close(&hnd);
     }
-    if (LTR_IsOpened(&m_loghnd.Channel)==LTR_OK) {
+    if (LTRLOG_IsOpened(&m_loghnd)==LTR_OK) {
         /* сперва закрываем соединение журнала со своей строны, затем ожидаем
            закрытия соединения со строны службы, чтобы не разрывать соедениение
            посередине передачи */
         INT ret = LTRLOG_Shutdown(&m_loghnd);
-        TLTRLOG_MSG *msg = NULL;
+        TLTRLOG_MSG *msg = Q_NULLPTR;
         QElapsedTimer tmr;
         tmr.start();
         while ((ret == LTR_OK) && (tmr.elapsed() < log_shutdown_tout)) {
@@ -194,12 +198,14 @@ int LtrdConnection::findCrate(QString serial, int intf /*= CRATE_IFACE_UNKNOWN*/
 }
 
 int LtrdConnection::processCrate(char *serial, int intf) {
-    TLTR_CRATE_STATISTIC stat;
+    TLTR_CRATE_STATISTIC stat;    
     int crate_err = 0;
     int new_crate = 0;
     int  fnd_ind = 0;
     LtrCrateInfo* fnd_info = 0;
 
+    memset(&stat, 0, sizeof(stat));
+
     fnd_ind = findCrate(serial, intf);
     if (fnd_ind >= 0) {
         fnd_info = m_crateList.at(fnd_ind);
@@ -221,8 +227,9 @@ int LtrdConnection::processCrate(char *serial, int intf) {
 
     for (int slot=0; !crate_err && (slot < stat.modules_cnt); slot++) {
         TLTR_MODULE_STATISTIC mstat;
+        memset(&mstat, 0, sizeof(mstat));
         mstat.mid = stat.mids[slot];
-        if ((mstat.mid != LTR_MID_EMPTY) && (mstat.mid!=LTR_MID_IDENTIFYING)) {
+        if (statisticAviableForMid(mstat.mid)) {
             crate_err = LTR_GetModuleStatistic(&hnd, fnd_info->m_intface, fnd_info->m_serial,
                                                slot+LTR_CC_CHNUM_MODULE1,
                                                &mstat, sizeof(mstat));
@@ -230,7 +237,9 @@ int LtrdConnection::processCrate(char *serial, int intf) {
                 mstat.mid = LTR_MID_EMPTY;
                 crate_err = 0;
             }
-        } if (!crate_err) {
+        }
+
+        if (!crate_err) {
             fnd_info->setModStat(&mstat, slot);
         }
     }
@@ -399,25 +408,60 @@ void LtrdConnection::checkCrates(void){
 }
 
 int LtrdConnection::readLog() {
-    int err = 0;
+    int err = LTR_OK;
     QMutexLocker locker(&m_logmutex);
     if (supportLog()) {
-        TLTRLOG_MSG *msg = NULL;
-        do {
-            err = LTRLOG_GetNextMsg(&m_loghnd, &msg, 2);
-            if (msg) {
-                QSharedPointer<LtrSrvLogRec> rec = QSharedPointer<LtrSrvLogRec>(
-                            new LtrSrvLogRec(msg));
-                m_logrecs.append(rec);
-                Q_EMIT newLogRecord(rec);
-                LTRLOG_FreeMsg(msg);
+        QDateTime checkDateTime;
+        /* открываем соединение чтения журнала, если оно не было открыто на данный момент */
+        if (LTRLOG_IsOpened(&m_loghnd) != LTR_OK) {
+            err = LTRLOG_Open(&m_loghnd, hnd.saddr, hnd.sport);
+            /* при повторной инициализации соединения запоминаем
+             * время последней считанной записи, чтобы избежать дублирования
+             * прочитанных записей с прошлым соединением */
+            if (!m_logrecs.isEmpty())
+                checkDateTime = m_logrecs.last()->time();
+        }
+
+        if (LTRLOG_IsOpened(&m_loghnd) == LTR_OK) {
+            QList< QSharedPointer<LtrSrvLogRec> > recs;
+            bool has_msg = false;
+
+            do {
+                TLTRLOG_MSG *msg = Q_NULLPTR;
+                err = LTRLOG_GetNextMsg(&m_loghnd, &msg, 2);
+                has_msg = msg != Q_NULLPTR;
+                if (msg) {
+                    QSharedPointer<LtrSrvLogRec> rec = QSharedPointer<LtrSrvLogRec>(
+                                new LtrSrvLogRec(msg));
+                    if (!checkDateTime.isValid() || (rec->time() > checkDateTime)) {
+                        recs.append(rec);
+                        m_logrecs.append(rec);
+                    }
+                    LTRLOG_FreeMsg(msg);
+                }
+            } while (has_msg && (err==LTR_OK));
+
+            if (!recs.isEmpty()) {
+                Q_EMIT newLogRecords(recs);
+
+                QList< QSharedPointer<LtrSrvLogRec> > remRecs;
+                while (m_logrecs.size() >  log_max_recs) {
+                    remRecs.append(m_logrecs.takeFirst());
+                }
+                Q_EMIT oldLogRecordsRemove(remRecs);
             }
-        } while (msg && (err==LTR_OK));
+        }
+
+        /* если произошла ошибка при чтении журнала, возможно мы его не успевали
+         * вычитывать и это не критическая ошибка.
+         * закрываем соединение, чтобы при следующей попытке чтения журнала
+         * его заново проинициализировать */
+        if (err!=LTR_OK) {
+            LTRLOG_Close(&m_loghnd);
+        }
     }
 
-    if (err!=LTR_OK) {
-        stopOnError();
-    }
+
     return err;
 }
 
@@ -484,10 +528,6 @@ void LtrdConnection::privStartConnection(bool restart) {
         err = LTR_Open(&hnd);
         if (err==LTR_OK)
             err = LTR_GetServerVersion(&hnd, (DWORD*)&m_vers);
-        if ((err==LTR_OK) && supportLog()) {
-
-            err = LTRLOG_Open(&m_loghnd, hnd.saddr, hnd.sport);
-        }
 
         m_status = err==LTR_OK ? STATUS_ONLINE : STATUS_ERROR;
         Q_EMIT statusChanged(m_status);
